import axios from 'axios';
import { API_BASE_URL } from '@/config/api-config'; // api-config.ts can be moved to src/config/
import router from '@/router'; // Import the router instance
import { useAuthStore } from '@/stores/auth'; // Import the auth store

// Create axios instance
const api = axios.create({
  baseURL: API_BASE_URL, // API_BASE_URL should come from env or config
  headers: {
    'Content-Type': 'application/json',
  },
  withCredentials: true, // Enable sending cookies and authentication headers
});

// Request interceptor
api.interceptors.request.use(
  (config) => {
    const token = localStorage.getItem('token'); // Or use useStorage from VueUse
    if (token) {
      config.headers.Authorization = `Bearer ${token}`;
    }
    return config;
  },
  (error) => {
    return Promise.reject(error); // Simpler error handling
  }
);

// Response interceptor
api.interceptors.response.use(
  (response) => response,
  async (error) => {
    const originalRequest = error.config;
    const authStore = useAuthStore(); // Get auth store instance

    if (error.response?.status === 401 && !originalRequest._retry) {
      originalRequest._retry = true;
      try {
        const refreshTokenValue = authStore.refreshToken; // Get from store for consistency
        if (!refreshTokenValue) {
          console.error('No refresh token, redirecting to login');
          authStore.clearTokens(); // Clear tokens in store and localStorage
          await router.push('/auth/login');
          return Promise.reject(error);
        }

        // Use the store's refresh mechanism if it already handles API call and token setting
        // However, the interceptor is specifically for retrying requests, so direct call is fine here
        // as long as it correctly updates tokens for the subsequent retry.
        const response = await api.post('/auth/jwt/refresh', {
          refresh_token: refreshTokenValue,
        });

        const { access_token: newAccessToken, refresh_token: newRefreshToken } = response.data;
        // The authStore.setTokens will update localStorage as well.
        authStore.setTokens({ access_token: newAccessToken, refresh_token: newRefreshToken });

        originalRequest.headers.Authorization = `Bearer ${newAccessToken}`;
        return api(originalRequest);
      } catch (refreshError) {
        console.error('Refresh token failed:', refreshError);
        authStore.clearTokens(); // Clear tokens in store and localStorage
        await router.push('/auth/login');
        return Promise.reject(refreshError);
      }
    }
    return Promise.reject(error);
  }
);

// Export the original axios too if some parts of your app used it directly
const globalAxios = axios;

export { api, globalAxios };

// Re-export apiClient for convenience, using the new api instance
// from src/config/api.ts
import { API_VERSION, API_ENDPOINTS } from '@/config/api-config';

export const getApiUrl = (endpoint: string): string => {
  // The API_ENDPOINTS already include the full path, so we just need to combine with base URL
  return `${API_BASE_URL}${endpoint}`;
};

export const apiClient = {
  get: (endpoint: string, config = {}) => api.get(getApiUrl(endpoint), config),
  post: (endpoint: string, data = {}, config = {}) => api.post(getApiUrl(endpoint), data, config),
  put: (endpoint: string, data = {}, config = {}) => api.put(getApiUrl(endpoint), data, config),
  patch: (endpoint: string, data = {}, config = {}) => api.patch(getApiUrl(endpoint), data, config),
  delete: (endpoint: string, config = {}) => api.delete(getApiUrl(endpoint), config),
};

export { API_ENDPOINTS }; // Also re-export for convenience